package dev.fitko.fitconnect.integrationtests;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.config.Environment;
import dev.fitko.fitconnect.api.config.EnvironmentName;
import dev.fitko.fitconnect.api.config.SenderConfig;
import dev.fitko.fitconnect.api.config.SubscriberConfig;
import dev.fitko.fitconnect.api.domain.model.event.problems.Problem;
import dev.fitko.fitconnect.api.exceptions.FileHandlingException;
import dev.fitko.fitconnect.client.SubscriberClient;
import dev.fitko.fitconnect.client.factory.ClientFactory;
import dev.fitko.fitconnect.core.util.FileUtil;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

import static dev.fitko.fitconnect.api.domain.model.event.problems.Problem.SCHEMA_URL;

public class IntegrationTestBase {

    private static final String authBaseUrl = "https://auth-testing.fit-connect.fitko.dev";
    private static final String routingBaseUrl = "https://routing-api-testing.fit-connect.fitko.dev";
    private static final String selfServicePortalUrl = "https://portal.auth-testing.fit-connect.fitko.dev";
    private static final String submissionBaseUrl = "https://submission-api-testing.fit-connect.fitko.dev";

    public static void cleanupTestSubmissions() {

        final UUID destinationId = UUID.fromString(System.getenv("TEST_DESTINATION_ID"));
        final Problem problem = new Problem(SCHEMA_URL + "technical-error", "cleanup", "submission-cleanup", "other");
        final SubscriberClient subscriberClient = ClientFactory.getSubscriberClient(getConfigWithCredentialsFromEnvironment("TEST", true));

        subscriberClient.getAvailableSubmissionsForDestination(destinationId).forEach((s -> {
            try {
                subscriberClient.rejectSubmission(s, List.of(problem));
            } catch (final Exception e) {
                //continue
            }
        }));
    }

    public static ApplicationConfig getConfigWithCredentialsFromEnvironment(final String environmentName, final boolean allowInsecurePublicKey) {

        final var sender = new SenderConfig(System.getenv("SENDER_CLIENT_ID"), System.getenv("SENDER_CLIENT_SECRET"));

        final var subscriber = SubscriberConfig.builder()
                .clientId(System.getenv("SUBSCRIBER_CLIENT_ID"))
                .clientSecret(System.getenv("SUBSCRIBER_CLIENT_SECRET"))
                .privateDecryptionKeyPaths(List.of("src/test/resources/private_decryption_test_key.json"))
                .privateSigningKeyPath("src/test/resources/private_test_signing_key.json")
                .build();

        final EnvironmentName envName = new EnvironmentName(environmentName);
        final Environment env = new Environment(authBaseUrl, routingBaseUrl, submissionBaseUrl, selfServicePortalUrl, allowInsecurePublicKey, false);

        return ApplicationConfig.builder()
                .senderConfig(sender)
                .subscriberConfig(subscriber)
                .environments(Map.of(envName, env))
                .activeEnvironment(envName)
                .submissionDataSchemaPath("submission-data-schema")
                .build();
    }

    public String loadContentOfFile(String filePath) {

        try {
            return Files.readString(new File(Objects.requireNonNull(FileUtil.class.getClassLoader().getResource(filePath)).getFile()).toPath());
        } catch (IOException e) {
            throw new FileHandlingException(e);
        }
    }
}
